/*
 * Copyright (c) 1983 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * From: @(#)io.c	5.6 (Berkeley) 3/1/91
 */
char io_rcsid[] = 
  "$Id: io.c,v 1.1 2001/03/09 12:59:09 vitas Exp $";

/*
 * This file contains the I/O handling and the exchange of 
 * edit characters. This connection itself is established in
 * ctl.c
 */

#include <sys/time.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include "talk.h"


#include <stdlib.h>

CTL_MSG msg;

int r_chars=0;
int l_chars=0;

int average= 0;


#define AVG_SIZ  61
#define AVG_DELTA 1 /* sec */
#define AVGMOD(idx) ((idx)%AVG_SIZ)

#define T1 5
#define T2 30
#define T3 60

#define PUTR statistika_put(r_avg, r_chars)
#define PUTL statistika_put(l_avg, l_chars)

#define GETR(back_time) statistika_get(r_avg, back_time)
#define GETL(back_time) statistika_get(l_avg, back_time)

/*
 * data pro logovani.
 */

#define LOG_BUFF_SIZ 66

#define I_SIGN "#"
#define U_SIGN "-"

char log_buff[2][LOG_BUFF_SIZ + 2];
int log_buff_idx[2]= {0,0};
FILE *log_file=NULL;


int last_time=0;

int r_avg[AVG_SIZ];
int l_avg[AVG_SIZ];

void statistika_init()
{
  int i;

  for(i=AVG_SIZ;i--;)
  {
    r_avg[i]=
    l_avg[i]= 0;
  }
} /* statistika_init */

  
void statistika_put( int *pole, int keystrokes)
{
  int i;
  int cur_time= time(NULL) / AVG_DELTA;
  
  if( last_time )
  {
    for(i= AVGMOD(last_time); i != AVGMOD(cur_time); i= AVGMOD(i+1) )
    {
      pole[i]= pole[AVGMOD(last_time)];
    }
  }

  last_time= cur_time;

  pole[AVGMOD(cur_time)]= keystrokes;
  
}

float statistika_get(int *pole, int back_time)
{
  int now, before;
  now=    pole[AVGMOD(last_time)];
  before= pole[AVGMOD(last_time + AVG_SIZ - back_time)];

  if(before == 0 )
  { /* jeste nic */
    return 0.0;
  }

  return  ((float)( now - before )) / ((float) (back_time * AVG_DELTA));
} /* statistika_get */


int score(char *buf, int len, WINDOW *w)
{
  int ret= 0;
  int x,y;
  getyx(w, y, x);
  for(;len--;buf++)
  {
    switch( *buf )
    {
    case 0x0c: break;
    case 0x7f: if(x){x--;ret--;} break;
    case 0x0d: x= 0;ret++;break;
    case 0x02: average= !average;wclear(line_win);*buf=' '; 
    default:   x++;ret++;if(*buf<' '){x++;ret++;}
    }
  }
  return ret;
} /* score */
/*
 * by vitas: loguj do souboru 
 */ 

void init_log()
{
  
  char *home= getenv("HOME");
  char  logfilename[150];

  struct tm *tm;
  time_t t;
  t= time(NULL);
  tm= localtime(&t);

  sprintf(
    logfilename, 
    "%s/.talk/%s.%02i%02i%02i.%02i%02i",
    home,
    msg.r_name,
    tm->tm_year%100,
    tm->tm_mon,
    tm->tm_mday,
    tm->tm_hour,
    tm->tm_min
    );

  log_file= fopen(logfilename, "a");
/* "/home/vitas/.talk/a", "a"); */
  if( log_file )
  {
    fprintf(
      log_file, 
      "%02i%02i%02i.%02i%02i %s("I_SIGN") talk with %s("U_SIGN")\n\n",
      tm->tm_year%100,
      tm->tm_mon,
      tm->tm_mday,
      tm->tm_hour,
      tm->tm_min,
      msg.l_name,
      msg.r_name
    );
  }
} /* init_log */ 

void flush_logbuf(int who)
{
  struct tm *tm;
  time_t t;

  if( ! log_file )
  {
    return;
  }
  t= time(NULL);
  tm= localtime(&t);

  log_buff[who][log_buff_idx[who]]= '\0';
  fprintf(
    log_file,
    "%02i:%02i:%02i%s%s\n",
    tm->tm_hour,
    tm->tm_min,
    tm->tm_sec,
    who?I_SIGN:U_SIGN,
    log_buff[who]
   );
  fflush(log_file);
} /* flush_log_buf */


  
  
void print2logbuf(char *buf, int len, int who)
{
  int *idx= log_buff_idx + who;
  for(;len--;buf++)
  {
    if(*buf == '\n' )
    {
       flush_logbuf(who);
       *idx= 0;
       continue;
    }
    log_buff[who][(*idx)++]= *buf == 0x7f? 0x8: *buf;
    if( *idx >= LOG_BUFF_SIZ )
    {
       flush_logbuf(who);
       *idx= 0;
       continue;
    }
  }
} /* print2logbuf */

  
  

      

/*
 * by vitas: print the midle line 
 */
void
print_midle_line()
{
	struct tm *tm;
	time_t t;
  	char *bra_rem, *bra_loc;
	char *bras="()[]{}<>!!??";
	int bras_last=strlen(bras)-2;
	int idx;

	t= time(NULL);
	tm= localtime(&t);

	bra_rem=
	bra_loc="()";

	if( r_chars > l_chars )
	{
		idx= ((r_chars+1)/(l_chars+1))*2;
		if( idx > bras_last) idx= bras_last  ;
		bra_rem=bras + idx;
	}
	else
	{
		idx= ((l_chars+1)/(r_chars+1))*2;
		if( idx > bras_last) idx= bras_last  ;
		bra_loc=bras + idx;
	}


	wprintw(
		line_win, 
		"\r[%2i:%02i:%02i] ",
		tm->tm_hour,
		tm->tm_min,
		tm->tm_sec
		);
	wprintw(
		line_win, 
		"domaci%c%s%c:%3i",
		bra_loc[0],
		msg.l_name,
		bra_loc[1],
		l_chars);
	if( average )
	{
	wprintw(
		line_win, 
		"[%4.1f,%4.1f,%4.1f]",
		GETL(T1),
		GETL(T2),
		GETL(T3)
		);
	}
	wprintw(
		line_win, 
		", hoste%c%s%c:%3i",
		bra_rem[0],
		msg.r_name,
		bra_rem[1],
		r_chars
		);
	if( average )
	{
	wprintw(
		line_win, 
		"[%4.1f,%4.1f,%4.1f]"
		,GETR(T1)
		,GETR(T2)
		,GETR(T3)
		);
	}
	wprintw( line_win, " ");
	wrefresh(line_win);
} 	/* print_midle_line */
/*
 * The routine to do the actual talking
 */
void
talk(void)
{
	fd_set read_set;
	int nb;
	unsigned char buf[BUFSIZ];

	message("Connection established");
	beep();
	wrefresh(curscr);

	init_log();

	current_line = 0;

	/*
	 * Wait on both the other process and standard input
	 */
	for (;;) {
		FD_ZERO(&read_set);
		FD_SET(0, &read_set);
		FD_SET(sockt, &read_set);
		nb = select(sockt+1, &read_set, NULL, NULL, NULL);
		if (nb <= 0) {
			if (errno == EINTR) {
				continue;
			}
			/* panic, we don't know what happened */
			p_error("Unexpected error from select");
			quit(0);
		}
		if (FD_ISSET(sockt, &read_set)) { 
			/* There is data on sockt */
			nb = read(sockt, buf, sizeof(buf));
			if (nb <= 0) {
				message("Connection closed. Exiting");
				quit(0);
			}
                        r_chars+= score(buf, nb, his_win.x_win);
                        if( average )
                        {       
                                PUTR;
                        }
                        print_midle_line(); 
                        print2logbuf(buf, nb, 1);
			display(&his_win, buf, nb);
		}
		if (FD_ISSET(0, &read_set)) {
			buf[0] = getch();
			nb = 1;

                        l_chars+= score(buf, nb, my_win.x_win);
                        if( average )
                        {       
                                PUTL;
                        }
                        print_midle_line(); 
                        print2logbuf(buf, nb, 0);

			/* might lose data here because sockt is nonblocking */
		        /* v: filter special chars */
 			/* print_out(&my_win, socket, buf, nb); */
			display(&my_win, buf, nb);
			write(sockt, buf, nb);
		}
	}
}

/*
 * p_error prints the system error message on the standard location
 * on the screen and then exits. (i.e. a curses version of perror)
 */
void
p_error(const char *string) 
{
	const char *rmcup, *smcup, *amsg="";

	rmcup = tigetstr("rmcup");
	smcup = tigetstr("smcup");

	if (rmcup || smcup) {
		amsg = ". Press any key...";
	}

	wmove(my_win.x_win, current_line%my_win.x_nlines, 0);
	wprintw(my_win.x_win, "[%s : %s (%d)%s]\n",
	    string, strerror(errno), errno, amsg);
	wrefresh(my_win.x_win);
	move(LINES-1, 0);
	refresh();
	if (rmcup || smcup) {
		/* alternative screen, prompt before exit */
		getch();
	}
	quit(0);
}

/*
 * Display string in the standard location
 */
void
message(const char *string)
{

	wmove(my_win.x_win, current_line%my_win.x_nlines, 0);
	wprintw(my_win.x_win, "[%s]\n", string);
	wrefresh(my_win.x_win);
}
